(* ::Package:: *)

(* ::Text:: *)
(*(* eudiffgeov4.m *)*)
(**)
(*(*This is a union of the old packages eudiffgeov3.m and riemann v2.m.  *)
(*Started 2017.1208.  Finished  2018.01.17. *)
(*Last Revision finished  2020.02.11.*)*)
(**)
(*(* Created by Rolf Sulanke *)*)
(**)
(*(* Contains Mathematica functions for applications to Euclidean Differential Geometry and*)
(*pseudo-Riemann geometry developed and described in the notebooks EuCurvesv4.nb,euklsfv4.nb,  RGv3.nb. *)*)
(**)
(*(* Warnings.  1.The contents of this package are tested with Mathematica version 11.2 and 12.0. To run the package with *)
(*earlier versions of Mathematica is not guaranteed. In particular  for applications to pseudo-Riemannian geometry, check wether the functions*)
(*{Grad, Symmetric, ZeroSymmetric, TensorSymmetry,SymmetrizedArray, TensorDimensions,Antisymmetric,Symmetrize}*)
(*are built-in in the used Mathemetica version; if not, as e. g. in version 8.0.1, use the first version RG.zip of this software.*)
(*2. Do not use this package or the packages euvecv2.m, neuvecv2.m, and tensalgv3.m  together with earlier versions of these packages; this may lead to confusion for Contexts and definitions!*)*)
(**)
(*(* Hint:s. *)
(**)
(*1. Be sure, that besides of this package also the packages euvecv2.m, *)
(*neuvecv2.m, and tensalgv3.m are contained in the working directory.*)
(**)
(*2. After importing this package define the number*)
(*dim (= dimension) and ind (=index) of the pseudo- Euclidean vector space as constants *)
(*within the Global context of the active notebook. *)
(*This number may be changed later in the notebook, if wanted. If it is defined before *)
(*importing the notebook it will be fixed for all the modules introduced in the packages!*)*)
(**)
(*(* Free software: Feel free to apply, distribute or change the package under your own name with the condition that you cite the original files with the Internet address or other source where you got it. I am not responsible for errors or damages originated by the use of the programs or functions programmed in my notebooks or packages; everybody who applies them should test carefully whether they are appropriate for his purposes.*)*)
(**)
(**)
(*(* Changes.*)
(*15.07.29. Exterior derivation ddd added.*)
(*16.01.19. RGT and rgT new definitions.*)
(*16,03.20. tv new definition.*)
(*16.10.26. d: Correction for f = constant.*)
(*17.02.18. curve: adapted (dim deleted; n=3.)*)
(*17.02.20. \[Sigma], \[Omega]: generalized to immersions.*)
(*17.02.20.ddd,dd: unified to dd.*)
(*17.02.24 cg corrected.*)
(*17.02.27. gauss:  Fastest function chosen.*)
(*17.03.03. meancurvature: Replaced by A. Gray's faster Definition.*)
(*17.03.29. Improvements of the definitions of the structure forms.*)
(*17.12.15. Definition of R is obsolet now, Use  RCg instead.*)
(**)*)
(*(*Changes at the union of the packages since 17.12.15:*)
(*R->RCg, rgT->rgt, tv, euclid -> eucl, lor, lorg.*)
(*New: symg.*)
(*18.01.04 tensorfield definition changed.*)
(*19.10.27. Module var amplified.*)
(*19.12.10. Normal vector nor of a surface introduced.*)
(*19.12.29. Differential d precised; Arguments can't be  Lists.*)
(*20.01.14.. Definition of dVol simplified.*)
(*20.01.29. geodcurvature corrected.*)
(*20.02.04. geodcurvature: Simplifications introduced.*)
(*20.02.05. normalcurvature korrigiert.*)
(**)*)


(* ::Section:: *)
(*    The usages *)


BeginPackage["eudiffgeov4`",{"tensalgv3`","euvecv2`","neuvecv2`"}] 


(* ::Subsection:: *)
(*Concepts for Immersions *)


circle2D::usage = "circle2D[t, r:1] is the parameter representation of the dircle in the plane with 
radius r and center at the origin.";

circ2D::usage = "circ2D[m, r:1][t] is the parameter representation of the 
dircle in the plane with radius r and center m."

spirgr::usage = "spirgr[t,a:1] is the parameter representation of the spiral 
tansformation group with parameter a in the Euclidean plane; a = 0 yields 
the group of rotattions around the origin.";

spiral::usage = "spiral[a:1,b:1][t] is a parameter representation of the logarithmic spiral 
in orthonormal coordinates of the Euclidean plane; a = 0 yields the circle of radius b with
center at the origin.";

curve::usage = "curve[t, z:x, n:3] is a curve in a space or manifold of dimension n given by its 
unspecified components {z[t][i]}. The default values of z is x and of n is 3. ";

tangent::usage= "tangent[curve][t] yields the tangent unit vector of curve[t].";

graph::usage ="graph[f][x,y,...] is the canonical parameter representation of the 
graph {x,y,...f[x,y,...]} of the function f, in arbitrary dimension."; 

arclength::usage="arclength[curve][ x] yields the arc length of the curve curve[x] as an indefinite
Integral; curve[x] must be given in orthonormal Cartesian coordinates.";

arc::usage = "arc[curve,x,y0,y] yields the length of the arc of the curve curve[x]for y0<x<y; 
curve[x] must be given in orthonormal Cartesian coordinates.";

curvature::usage = "curvature[curve][t] yields the absolute value of the curvature 
of the regular curve[t] in any dimension, as a function of an arbitrary parameter t.";

curvaturevector::usage = "curvaturevector[x][t] is the curvature vector of the curve x[t] 
in the Euclidean n-Space, t an arbitrary parameter.";

torsion::usage = "torsion[curve][t] yields the torsion of the curve[t] in 
the Euclidean 3-space; it is defined only at points with non vanishing curvature.";

frenet::usage = "frenet[curve][t] yields the Frenet frame for a generally curved 
curve[t] in the n-dimensional Euclidean space. Generally curved means that the 
rank of the first n-1 derivatives of curve[t] must be n-1 at each point t.";

curvatures::usage = "curvatures[curve][t] yields the n-1 curvatures of a generally curved 
curve[t] in the n-dimensional Euclidean space. Generally curved means that the 
rank of the first n-1 derivatives of curve[t] must be n-1 at each point t.";

assu::usage ="assu[c][s] is a set of assumptions in the first four derivatives of the 
parameter representation c[s] of a smooth curve in the n-dimensional Euclidean space 
giving True if s is the Euclidean arc length of the curve.";

fr::usage ="fr[i][f][u,v], i = 0,1,2,3, is the i-th column of the matrix 
frame[f][u,v]. In particular fr[0] represents the point f[u,v] and fr[i] 
the i-th vector of the adapted orthonormal frame of the surface.";

frame::usage = "frame[f][u,v] is the element of the Euclidean group 
transforming the standard orthonormal frame {stb[i]}, i= 1,2,3,and 
the origin [0,0,0} into the adapted moving frame of the surface f[u.v].";

d::usage="For functions f[u,...] of m variables u,..., with values in an 
n-dimensional manifold or space d[f][u,...] is the differential of 
the function. The differentials d[u][u,...] for the 
variables u,..., themselves yield the standard cobasis of the dual 
of the tangent space at the point {u,...} of the parameter manifold.";

(*d::usage = "d[f][u,...] or d[f][{u,...}]is the differential of the function f[u,...] of several variables 
with values in an n-dimensional manifold or space. For the variables u, ... the differentials 
d[u][u,v,...], ... yield the standard cobasis in the cotangent space of the parameter 
manifold at the point {u, ...}. d[f][u,..].vl is the value of the differential d[f][u,...] 
for the tangent vector vl."; *)

jac::usage = "jac[f][x1,...] or jac[f][{x1,...}] is the transposed Jacobian matrix of a map f of the m-dimensional 
parameter manifold with coordinates {x1,...,xm) into the n-dimensional space if n>1, and d[f][x] if n==1.";


identity::usage = "identity[u1,...,um] ={u1,...,um} is the identity function on the 
manifold U with coordinates u1,...,um.";

tangsp::usage = "tangsp[f][u,...][x,...] is the tangent vector with standard 
coordinates x,... of the immersion f[u,...] at the point {u,...}.";

tv::usage = "tv[f][i][u,v,...]is the tangent vector of the i-th  parameter line of 
the immersion f[u,v,...] at the point {u, v,...}. If this point is regular these 
vectors for i = 1,...,Length[{u,v,...}] form a basis of the tangent space 
image[d[f][u,v,...]].";

vv::usage = "vv[a][x1,...,xm] is a vector field on the m-manifold with 
coordinates  {x1,...,xm} with components a[i][x1,...,xm], i = 1,...,m.";

tvf::usage = "tvf[a][f][u,v,...] is the tangent vector field on the 
m-surface f[u] with coordinates a[i][u,v,...] with respect to the 
standard basis tv[f][i][u,v,...].";

nor::usage = 
  "nor[f_][u_,v_] is the unit normal vector of the oriented surface \
f[u,v] in the Euclidean 3-space at the point f[u,v]"; 

rgt::usage = "For the immersion f of an m-dimensional manifold \!\(\*SuperscriptBox[\(X\), \(m\)]\) into 
the n-dimensional Euclidean space, m<=n,  rgt[f][u1,...,um][i,j], 1<=i,j<=m, is the (i,j)-coordinate of the 
Riemannian metric on  \!\(\*SuperscriptBox[\(X\), \(m\)]\) induced by the immersion f.";

RGT::usage = "RGT[f][u,v,...]is the matrix of the coordinates of the first fundamental 
form of the immersion f[u,v,...] into the n-dimensional Euclidean space: Its Riemannian inner metric.";

fund1::usage = "fund1[f][u1,..,um][x,y] is the value of the first fundamental form 
of the immersion f[u1,...,um] into the n_dimensional Euclidean space, 
for the tangent vectors x,y of the immersion.";

dVol::usage = "dVol[\[Gamma]\[Gamma]][u,v,...] is the volume element of the pseudo-Riemannian manifold 
with scalar product \[Gamma]\[Gamma]. See also dV,dA.";

dV::usage = "dV[f][u,v,...] is the volume element of the immersion f[u,v,...] in 
the Euclidean Space.. See also dVol,dA.";



(* ::InheritFromParent:: *)
(**)


(* ::Subsection:: *)
(*Concepts for Surfaces *)


tplane::usage = "tplane[f][uo,vo][x,y] is a parameter representation of the tangent 
plane of the surface f[u,v]in the affine n-space at the point {uo, vo} with parameters x,y.";

amf::usage = "amf[i][f][u,v]is the i-th vector, i = 1,2,3, of the orthonormal moving 
frame adapted to the parameter representation of the surface f[u,v] in the 
Euclidean 3-space.";

dA::usage = "dA[f][u,v] is the area element of the surface f[u,v] in the 
Euclidean n-space.";
(*
dd::usage = "If h[u,v] is a differential form in two variables u,v or a vector of such 
differential forms then dd[h][u,v] is its exterior differential."; *)

\[Sigma]::usage = "{\[Sigma][i][f][u1,..,um]}, i = 1,...,n, is the cobasis field to the orthonormal basis field 
in the tangent bundle of the immersion f[u,...]of an m-dimensional manifold into the n-dimensional Euclidean space, 
\[Sigma][i][f][u1,.,um]= {0,..,0} for m<i<= n. ";

\[Omega]::usage = "\[Omega][i,j][f][u,...] is the (i,j)-th structure form in the derivation equations 
of the immersion f[u,...]into the n-dimensional Euclidean space.For n=3 one sets \[Omega][1]=\[Omega][2,3], \[Omega][2]=\[Omega][3,1], \[Omega][3]=\[Omega][1,2].";

gauss::usage = "gauss[f][u,v] is the Gaussian curvature of the surface f[u,v]
in the Euclidean 3-space.";

normalcurvature::usage = "normalcurvature[ff,hu,hv][t]is the normal curvature of 
the curve ff[hu[t],hv[t]], t an arbitrary parameter.";

prcurvatures::usage = "prcurvatures[f][u,v] yields the principal curvatures of 
the surface f[u,v] in the Euclidean 3-space.";

meancurvature::usage = "meancurvature[f][u,v] is the mean curvature of 
the surface f[u,v] in the Euclidean 3-space.(A. Gray)";

fd2::usage = "{fd2[i,j][f][u,v], i,j = 1, 2} is the matrix of the coordinates  
with respect to the local standard basis {du,dv} of the second fundamental form 
of the surface f[u,v] in the Euclidean 3-space.";

pluckerconoid::usage = "pluckerconoid[u,v] is a parameter representation of Pl\[UDoubleDot]cker's conoid.";

conesf::usage = "conesf[y][u,v] is the general conical surface with vertex at the origin. 
It is defined by the  curve y[v] in the space of dimenion dim; dimmust be defined.";

cylindersf::usage = "cylindersf[x,y][u,v] is the cylinder with rulings parallel to 
the z-axis erected over the curve {x[v],y[v],0} in the x,y-plane.";

tangsf::usage = "tangsf[x][u,v] is a parameter representation of the developable 
surface formed by the tangents of the space curve x[v].";

cg::usage= "cg[i][f][u,v], i = 1,2, are the coordinates in the decomposition of 
\[Omega][3][f][u,v] with respect to the cobasis {\[Sigma][i][f][u,v]}."

fund2::usage= "fund2[f][u,v][xx,yy] is the value of the second fundamental form of 
the surface f[u,v] in the Eucliddean 3-space for the tangent vectors xx,yy.";

cf::usage = "cf[i,j][f][u,v] are the coordinates  of the second fundamental form 
with respect to the cobasis {\[Sigma][i][f][u,v]} of the surface f[u,v] in the Euclidean 3-space.";

normals::usage = "normals[f,u,v][s] is the normal congruence of the surface f[u,v]
in the Eucidean 3-space; s the line parameter.";


(* ::Subsection:: *)
(*Exterior Differential dd *)


dd::usage = "dd[ta][x,...] yields the exterior differential of the p-form ta[x,...];
remember:a 0-form ta[x...] must be a function with the Attribute NumericFunction; 
for antisymmetric covariant tensors ta[x,...] the operator dd returns a covariant tensor, 
and for an antisymmetric Array ta[x,...] it returns the derived Array; all dimensions 
of the Array ta[x,...] must equal the number of the variables x,...";


(* ::Subsection:: *)
(*Manifolds*)


var::usage = "If Head[x]== Symbol, var[x, n:dim] is the sequence of the 
coordinates x[1],...,x[n] in a chart, n = dim. If Head[x]== List, 
var[x] is the sequence of the elements of x. All other arguments yield 
Null and an error message.";

\[Delta]::usage = "\[Delta][i,j] is the short form of KroneckerDelta[i,j].";

imm::usage = "imm[f][x1,...,xm] is the local coordinate representation of an unspecified smooth map 
 of an m-dimensional parameter manifold into an n-dimensional smooth manifold, n = dim must 
be  defined in the global context. If the rank of the differential d[imm[f]][x,...] is maximal, 
i.e. equals min[m,n], at all points, the map is said to be an immersion, if m\[LessEqual] n, a submersion, 
if m\[GreaterEqual]n, and a local diffeomorphism if m = n.";

tx\[Iota]::usage = "The local vector field tx\[Iota][i][x[1],...,x[n]] yields the i-th vector of the standard basis 
corresponding to a chart \[Phi]\[Iota] of the manifold \!\(\*SuperscriptBox[\(M\), \(n\)]\) at the points x 
with coordinates {x[1],...,x[n]}."; 

der::usage = "der[vlist,h][x1,...,xn] is the directional derivative of the function h[x1,...,xn] in 
direction of the tangent vector with coordinates vlist, calculated in local coordinates 
with the local representative of h.";

comm::usage ="comm[X,Y][x1,...,xn] yields the commutator of the local vector fields X[x1,...,xn],Y[x1,...,xn] 
on a differentiable manifold.";

tensorfield::usage ="tensorfield[a,p,q][x1,..,xn] yields a tensor of type (p,q)at the point with 
coordinates {x1,..,xn} whose tensor coordinates are a[ii,jj][x1,..,xn]. 
In case of a function, p=q=0, the argument a must be the name a of the function defined
in the shape a[x1,..,xn]. ";


(* ::Subsection:: *)
(*Linear Connections*)


\[CapitalGamma]::usage = "\[CapitalGamma][k,i,j][x,y,...] denotes the \[CapitalGamma]-symbols of a general unspecified linear connection.";

\[CapitalGamma]g::usage = "\[CapitalGamma]g[g][k,i,j][x,y,...] are the Christoffel symbols of the second kind of the 
pseudo-Riemannian metric {g[x,y,...][i,j]} in local coordinates x,y,...";

cnf::usage = "cnf[\[CapitalGamma]][i,j][x][v]evaluates the connection form with indices (i,j) of the linear 
connection \[CapitalGamma] for tangent vectors v at the point x of the manifold. x and v must be entered by 
their coordinates.";

nabla0::usage ="nabla0[f][x1,...,xn] is the tensor field of type (1,0) corresponting to the differential of the 
function f[x1,...,xn] of the local coordinates (x1,...,xn),"; 

nabla1::usage = "nabla1[\[CapitalGamma],Y][x1,...,xn] is the  tensor field of type (1,1), named the gradient tensor, 
of the vector field Y[x1,...,xn] in local coordinates (x1,...,xn)."; 

nab1::usage = "nab1[\[CapitalGamma],Y][x1,...,xn][h,i] is the (h,i)-coordinate of the gradient tensor field 
nabla1[\[CapitalGamma],Y][x1,...,xn] of the vector field Y[x1,...,xn] in local coordinates (x1,...,xn).";

nab::usage = "nab[\[CapitalGamma], th][x][ii,jj] is the coordinate of the gradient tensor of the tensor th[x] 
with respect to the linear connection \[CapitalGamma] corresponding to the index tuples ii, jj.";

nabla::usage ="nabla[\[CapitalGamma], th][x] yields the gradient tensor of the tensor field th[x] with respect 
to the linear connection \[CapitalGamma].";

absD::usage = "absD[\[CapitalGamma], ta][x][X][ii,jj] yields the coordinate of the absolute differential of the 
tensor field ta[x] at the tangent vector X or vector field X[x] with respect to the linear 
connection  \[CapitalGamma] corresponding to the index tuples ii,jj.";

tensorD::usage = "tensorD[\[CapitalGamma], ta][x] is the absolute differential of the tensor field ta[x] 
with respect to the linear connection \[CapitalGamma]; tensorD[\[CapitalGamma],ta][x][X] evaluates the absolute differential 
at the tangent vector X or the vector field X[x].";

absD1::usage = "absD1[\[CapitalGamma], ta][cu,t][ii,jj] yields the coordinate of the absolute differential of the 
tensor field ta[t] for the tangent vector cu'[t] along the curve cu[t] with respect to 
the linear connection  \[CapitalGamma] corresponding to the index tuplesii,jj.";

tensorD1::usage = "tensorD1[\[CapitalGamma], ta][cu,t] is the absolute differential of the tensor field ta[t] along 
the curve cu[t] with respect to the linear connection \[CapitalGamma] evaluated for the tangent vector cu'[t] of the curve.";

tors::usage = "tors[\[CapitalGamma]}[x1,...,xn] is the torsion tensor field of a linear connection \[CapitalGamma] on a 
n-dimensional manifold.";

RC::usage ="RC[\[CapitalGamma]][x1,...,xn][k,i,h,j]is the component of the curvature tensor of the linear 
connection \[CapitalGamma] with indices k (contravariant) and h,i,j (covariant) in a n-dimensional smooth manifold.";

RCtensor::usage = "RCtensor[\[CapitalGamma]][x1,...,xn] is the curvature tensor (of type (3, 1)) of the linear 
connection \[CapitalGamma] on a smooth n-dimensional manifold. Its coordinates are RC[\[CapitalGamma]][x1,...,xn][k,i,h,j].";

Riccitensor::usage = "Riccitensor[\[CapitalGamma]][x1,...,xn] is the Ricci curvature tensor of the linear connection \[CapitalGamma].";

rct::usage ="rct[\[CapitalGamma]][x1,...,xn][i,j] is the (i,j)-coordinate of the Ricci curvature tensor of the linear connection \[CapitalGamma].";


(* ::Subsection:: *)
(*Riemannian Geometry*)


RG::usage = "RG[gg:g][x1,...] is the matrix of the elements gg[x1,...][i,j] of a symmetric matrix 
field on the manifold with coordinates {x1,...};
the default argument gg=g gives a general,unspecified symmetric matrix field.";

symg::usage =" symg[h][x,y,...][i,j] is the element h[x,y,...][i,j] of a  symmetric matrix field."

rg::usage = "rg[gg_:g][x__][i,j] is the (i,j)-element of the symmetric matrix RG[gg][x].";

RGtensor::usage = "RGtensor[gg][x,y,...] is the covariant symmetric tensor field of 
degree 2 whose coordinate Array is RG[gg][x,y,...].";

INV::usage ="If m[u,v,...] is an invertible square matrix of functionsf[u,v,...][i,j] 
then INV[m][u,v..] is the inverse matrix.";

inv::usage = "inv[m][u,v,...][i,j] is the (i,j)-element of the matrix INV[m][u,v,...].";

eucl::usage = "eucl[x1,...,xn][i,j] =\[Delta][i,j]are the coordinates of the n-dimensional 
Euclidean metric tensor in orthonormal coordinates.";

EUCL::usage =  "EUCL[x1,x2,...,xn]is the matrix of the coordinates of the 
n-diemensional Euclidean metric with respect to orthonormal Cartesian coordinates.";

lor::usage = "lor[k:ind][i,j] is the standard pseudo-orthonormal form of the metric 
of the pseudo-Euclidean vector space of index k with default k=ind;i,j = 1,...,dim.";

lorg::usage = "lorg[k:ind][x1,...,xn][i,j] is the standard pseudo-orthonormal form of the 
pseudo-Riemannian metric of the pseudo-Euclidean space of index k with default 
k=ind;i,j = 1,...,n=dim.";


 RGposk::usage = "RGposk[r][s1,s2,...,sn]  is the matrix of the coordinates of the 
n-dimensional Riemannian metric of positive constant curvature 1/r^2 with respect 
to the spherical coordinates (s1,...,sn) used in the definition of unit.";

rgposk::usage = "rgposk[r][s1,s2,...,sn][i,j] is the (i,j)-coordinate of 
RGposk[r][s1,s2,...,sn].";

rgt::usage = "rgt[f][x,y,...][i,j]is the (i,j)-coordinate of the inner Riemannian 
metric of the immersion f[x,y,...] into an Euclidean space.";

rgT::usage = "rgT is obsolete, use rgt instead.";


chr::usage = "chr[i,j,k,g:rg[]][x,y,...] are the Christoffel symbols of the 
first kind of the pseudo-Riemannian metric {g[i,j][x,y,...]} in local 
coordinates x,y,... The default g is the unspecified symmetric gr.";

\[CapitalGamma]g::usage = "\[CapitalGamma]g[g:rg[]][k,i,j][x,y,...] are the Christoffel symbols of the second kind 
of the pseudo-Riemannian metric {g[x,y,...][i,j]} in local coordinates x,y,...  
\[CapitalGamma]g[ ][k,i,j][x,y,...] yields the Christoffel symbpls of the default unspecified metric rg.
See also \[CapitalGamma].";

\[CapitalGamma]f::usage ="\[CapitalGamma]f[ff][h,i,j][u,v,...], h,i,j = 1,...,Length[{u,v,...}], are the Christoffel 
symbols of the second kind of the Riemannian inner metric  of the immersion ff[u,v,...] 
into the Euclidean n-space.";

\[CapitalGamma]0::usage = "\[CapitalGamma]0[[i,j,k]x,...] (= 0 in Cartesian coordinates) are the \[CapitalGamma]-symbols of the
 flat linear connection.";

covRCtensor::usage = "covRCtensor[gg:rg[]][x,y,...] is the covariant curvature 
tensor field of the pseudo-Riemannian space with metric tensor {gg[x,y,...][i,j]}.";

covrc::usage = "covrc[[gg:rg[]][k,h,i,j][x,y,...] are the coordinates of the covariant 
curvature tensor covRCtensor[gg:rg[]][x,y,...].";

sectionalcurvature::usage = "sectionalcurvature[gg:rg[]][x,y,...][X, Y] calculates 
the sectional curvature for the tangent 2-space span[X,Y] at the point {x,y,...} of 
a pseudo-Riemannian manifold with metric tensor gg.";

tensorscpr::usage = "For tensors ta,tb of the same type (p, q) over a finite dimensional 
vector space with a scalar product g with coordinates g[i,j] the function 
tensorscpr[g][ta,tb] is the value of the tensor scalar product for the tensors ta,tb.";

tensorfieldscpr::usage = "tensorfieldscpr[gg:rg[]][ta,tb][x,y,...] calculates the 
tensor scalar product of the tensor fields ta, tb of the same type
(p, q) over a pseudo-Riemannian manifold with the metric gg[x,y,..][i,j] 
and coordinates x,y,...";

spher::usage = "spher[r][s1,...,sn] is a parameter representation of the n-dimensional 
sphere of radius r with center at the origin of the (n+1)-dimensional Euclidean space.";

sph::usage  = "sph[r][x1,...,xn] is a parameter representation of the n-sphere of 
radius r with center at the origin  of the (n+1)-dimensional Euclidean space, defined 
by the inverse stereographic projection.";

RGpos::usage = "RGpos[r][s1,...,sn] is the matrix of the coordinates of the inner 
Riemannian metric (first fundamental form) of the n-dimensional sphere of 
radius r>0 in spherical cordinates s1,...,sn.";

RGcnfpos::usage ="RGcnfpos[r][x1,...,xn] is the matrix of the coordinates of 
the Riemannian metric of positive curvature 1/r^2 in the conformal model.";

rgcnfpos::usage="rgcnfpos[r][i,j][x1,...,xn] is the (i,j)-coordinate of the 
Riemannian metric tensor of constant positive curvature 1/r^2 in the 
conformal model.";

hypsp::usage = "hypsp[r][s1,...,sn] is a parameter representation of 
the pseudo-Euclidean model of the n-dimensional hyperbolic space
with curvature -1/r^2.";

RGhypsp::usage = "RGhypsp[r][s1,s2,...,sn] is the metric tensor 
of the n-dimensional hyperbolic space of constant curvature 
-1/r^2 given by its components with respect to the coordinates o
f the pseudo-Euclidean model.";

rghypsp::usage = "rghypsp[r][s1,s2,...,sn][i,j] is the (i,j)-
coordinate of RGhypsp[r][s1,s2,...,sn].";

hypcnf::usage = "hypcnf[r][s1,...,sn] for s1^2+...+sn^2<1 is a 
parameter representation of the conformal model of the n-dimensional 
hyperbolic space with curvature -1/r^2.";

RGhypcnf::usage = "RGhypcnf[r][s1,s2,...,sn] is the metric tensor of the 
n-dimensional hyperbolic space of constant curvature -1/r^2 given by 
its components with respect to the coordinates of the spherical conformal model.";

rghypcnf::usage = "rghypcnf[r][s1,s2,...,sn][i,j] for s1^2+...+sn^2<1 is 
the (i,j)-coordinate of RGhypcnf[r][s1,s2,...,sn].";

RGneg::usage = "RGneg[r][s1,s2,...,sn] for s1^2+...+sn^2<1 is the metric 
tensor of the n-dimensional hyperbolic space of constant curvature -1/r^2 
in its n-disk conformal model.";

rgneg::usage = "rgneg[r][s1,s2,...,sn][i,j] for s1^2+...+sn^2<1 is 
the (i,j)-coordinate of RGneg[r][s1,s2,...,sn].";

X\[Iota]::usage = "X\[Iota][v][x[1],...,x[n]] is a local vector field defined 
on a chart domain  \!\(\*SubscriptBox[\(U\), \(\[Iota]\)]\) with vector coordinates given by the n-vector v.";


invRG::usage = "If {g[u,v,...][h,k]}is a field of square matrices the rank of 
which equals the number of variables u,v,..., then invRG[g][u,v,...] is the field 
of the inverse matrices.";

inverse::usage = "If {g[u,...][h,k]}is a field of square matrices defining a 
pseudo-Riemannian geometry, then {inverse[g][u...][i,j]} is the field of 
the inverse matrices.";

invrgT::usage = "invrgTf][u,v,...][i,j][ is the (i,j)-coordinate of the inverse first 
fundamental form of the immersion f[u,v,...].";

pT::usage = "For the immersion f[u,v,...] into the Euclidean n-space the function  
pT[[f][u,v,...][vx]is the orthogonal projection of the vector vx (its tangential component) 
at the point  f[u,v,...].";

christoffel1::usage = "christoffel1[i,j,k][gg][x,y,...] are the Christoffel symbols of the 
first kind of the pseudo-Riemannian metric {gg[x,y,.][i,j]} in local coordinates x,y,...";

\[Gamma]\[Gamma]::usage = "\[Gamma]\[Gamma][j,k,i][f][x,y,...]  are the Christoffel symbols of the first kind of the 
Riemannian metric rgt[f][x,y,...] of the immersion f[x,y,...] into the Euclidean n-space.";

\[CapitalGamma]::usage = "\[CapitalGamma][k,i,j][x,y,...] denotes the \[CapitalGamma]-symbols of a general unspecified linear connection.";

\[CapitalGamma]rgT::usage = "\[CapitalGamma]rgT[j,k,i][f][x,y,...]  are the Christoffel symbols of the second  kind of the 
Riemannian metric rgt[f][x,y,...][i,j] of the immersion f[x,y,...] into the Euclidean n-space.";

euclid::usage = "euclid[x,y,...][i,j] denotes the i,j-component of the Riemannian metric 
of the Euclidean n-space in orthonormal Cartesian coordinates.Obsolete, use eucl instead.";

DD::usage = "DD[g][vw][x,y,...] is the absolute Differential of the vector field vw[x,y,...] on 
a pseudo-Riemannian manifold with metric tensor g[i,j][x,y,...].";

DDimm::usage = "DDimm[[f][vw][x,y,...] is the absolute Differential of the vector field vw[x,y,...] 
on an immersion f[x,y,...]into the Euclidean n-space; here vw is a vector field on the 
parameter manifold. and the images of DDim lie in the Euclidean n-space.";

DD1::usage = "DD1[ff,cu,vw][t] is the absolute derivative of a vector function 
vw[t] ={vw1[t],vw2[t]} defined along a curve with local coordinatess cu[t]={u[t],v[t]} 
of a surface ff[u,v] in the n-dimensional Euclidean space. For higher dimensional 
immersions or Riemannian spaces create a new Module changing the definition of \[CapitalGamma]\[CapitalGamma] 
corresponding to the situation.";

geodcurvature::usage ="geodcurvature[f_,hu_,hv_,pr_:False][t_] is the geodesic curvature of the curve 
f[hu[t],hv[t]] on the surface f[u,v] in the Euclidean 3-space, t an arbitrary parameter;
settting pr -> True plots some details of the evaluation, the default value is False.";

R::usage ="R[k,h,i,j][g][x,y,...] are  the coordinates of the Riemann curvature tensor of the pseudo-Riemannian 
space with metric tensor g[i,j][x,y,...]. R is obsolet now, use RCg instead.";

(*RCg::usage ="RCg[g][x,y,...][k,h,i,j] are  thre coordinates of the Riemann curvature tensor of the pseudo-Riemannian 
space with metric tensor g[i,j][x,y,...],with contravariant index k and covariant indices h,i,j.";*)

Rimm::usage ="Rimm[k,h,i,j][f][x,y,...] are the coordinates of the Riemann curvature tensor of the 
immersion f[x,y,...] into the Euclidean n-space.";

covR::usage ="covR[i,j,k,l][g][x,y,...] are the coordinates of the covariant form of the Riemannian curvature tensor 
of the pseudo-Riemannian metric g[x,y,.][i,j].";

covRimm::usage ="covRimm[i,j,k,l][f][x,y,...] are the coordinates of the covariant form of the Riemannian curvature tensor
 of the immersion f[x,y,...] into the n-dimensional Euclidean space.";


(* ::Section:: *)
(*The Definitions*)


(* ::Subsection:: *)
(*Euclidean Differential Geometry*)


(* ::Subsubsection::Closed:: *)
(*Curves*)


Begin["`Private`"] 

dimm := Global`dim; 

indd := Global`ind;



(* The spiral group in the plane *)

spirgr[t_,a_:1]:={{Cos[t],-Sin[t]},{Sin[t],Cos[t]}}*Exp[a*t];

spiral[a_:1, b_:1][t_] := spirgr[t,a].{b,0};

curve[t_,z_:Global`x,n_:3] := vec[z[t],n];

tangent[curv_][t_]:= Simplify[dotnorme[D[curv[t],t]], t\[Element] Reals];

(* The graph of a function is formally very easy to define: *)

graph[f_][x___] := {x, f[x]};


arc[cur_,x_,y0_,y_]:= Simplify[Integrate[norm[cur'[x]],{x,y0,y}]];

arclength[cur_][x_]:= Simplify[Integrate[norm[cur'[x]],x]]; 

curvature[cur_][t_] :=norm[ D[tangent[cur][t],t]]/norm[D[cur[t],t]];

curvaturevector[cur_][t_] :=  D[tangent[cur][t],t]/norm[D[cur[t],t]];

torsion[cur_][t_] := Det[{D[cur[t],t], D[cur[t],{t,2}],D[cur[t],{t,3}]}]/norm[Cross[D[cur[t],t], D[cur[t],{t,2}]]]^2;

frenet[cur_][t_] := Module[{n = Length[cur[t]], v,i,w},
v = Table[D[cur[t],{t,i}],{i,1,n-1}];
w = outzero[Simplify[Orthogonalize[v,Dot]/.Abs[vv_]^2 -> vv^2,TimeConstraint -> Infinity]];
If[Length[w]<n-1, Print[cur[t] , "is not generally curved!"]; Return[]];
v= Append[w, cross[w]]; v];

curvatures[cur_][t_]:= Module[{n = Length[cur[t]],cv, cc,i},
cv = frenet[cur][t];
If[cv == Null, Return[]];
For[i=1, i<n,i++,cc[i] = D[cv[[i]],t].cv[[i+1]]/norm[D[cur[t],t]]];
vec[cc,n-1]];

assu[c_][t_] :=(c'[t].c'[t]== 1&&c'[t].c''[t]==0&&-c'[t].c'''[t]==c''[t].c''[t]&&
-Derivative[1][c][t].Derivative[4][c][t]==3*c'''[t].c''[t]);

circ2D[m_List,r_:1][t_]:= m+r*{Cos[t],Sin[t]};

circle2D[t_, r_:1] := {Cos[t],Sin[t]}*r;


(* ::Subsubsection::Closed:: *)
(*Immersions. The Differential *)


Attributes[d]={Listable};
d[f_][x___]:= Module[{m=Length[{x}],u,v,i},
If[m==0,Print["Function ",f," needs arguments!"]; Return[]];
If[MemberQ[{x},f], Return[D[{x},f]]];
For[i=0,i<=m,i++,If[Head[{x}[[i]]]==List,
Print["Argument ",{x}[[i]]," is not a variable!"]; Return[]]];
D[f[x],{{x}}]];

(*Attributes[d]={Listable};
d[f_][x___]:= Module[{m=Length[{x}], u,v,i},
If[m==0,Print["Function ",f," needs arguments!"]; Return[]];
If[MemberQ[{x},f], Return[D[{x},f]]];
If[(m==1)&&Not[Head[x]===List],Return[D[f[v],v]/. v->x]];
If[m>=2,For[i=1,i<=m,i++,If[Head[{x}[[i]]]==List,
Print["Argument ",{x}[[i]]," is not a variable!"]; Return[]]]];
D[f[x],{{x}}]] *)


d[f_][{x__}]:= Module[{ff,n = Length[{x}],y},
If[MemberQ[{x},f], Return[D[{x},f]]];
ff[y__]:=f[{y}];
d[ff][x]];


jac[f_][x__] := If[(Length[{f[x]}]==1)&&Not[Head[f[x]]=== List],d[f][x],Transpose[d[f][x]]];

tangsp[f_][u__][x__]:=If[Not[Length[{x}]== Length[{u}]],
Print["invalid arguments!"];Return[],Return[d[f][u].{x}]];
  
(* Standard tangent basis *)
(*
tv[i_][f_][u__]:= If[(Head[f[u]]== List)&& (Length[f[u]]>= Length[Flatten[{u}]])&&
 MemberQ[Range[ Length[Flatten[{u}]]],i],Transpose[d[f][u]][[i]],
Print["Invalid arguments!"],Print["Invalid arguments!"]]; *)

tv[f_][i_][u__]:= If[(Head[f[u]]== List)&& (Length[f[u]]>= Length[Flatten[{u}]])&&
 MemberQ[Range[ Length[Flatten[{u}]]],i],If[Length[{u}]==1,Return[d[f][u]],Return[Transpose[d[f][u]][[i]]]],
Print["Invalid arguments!"],Print["Invalid arguments!"]];


nor[f_][u_,v_]:= dotnorme[Cross[tv[f][1][u,v],tv[f][2][u,v]]]/;(Head[f[u,v]] == List&&Length[f[u,v]]== 3)


identity:={##}&;

vv[a_]:=Table[a[i][##],{i,Length[{##}]}]&;

tvf[a_][ff_]:= Sum[tv[ff][i][##]*a[i][##],{i,Length[{##}]}]&;



(* ::Subsubsection:: *)
(* Riemannian metric induced by an immersion *)


invRG[g_][u__]:= Module[{m =Length[{u}],k,h,mM},
mM = Table[g[u][h,k],{h,m},{k,m}];
Inverse[mM]];

inverse[g_][u__][i_,j_]:= invRG[g][u][[i,j]];

invrgT[ff_][u__][i_,j_]:= Module[{gg,m =Length[{u}],k,h,v},
gg[v__][h_,k_] := rgt[ff][v][h,k] ;
inverse[gg][u]][i,j];

pT[ff_][x__]:= 
Sum[tv[ff][k][x]*Sum[invrgT[ff][x][k,j]*(tv[ff][j][x].#),{j,
Length[{x}]}],{k,Length[{x}]}]&;

geodcurvature[ff_,hu_,hv_,pr_:False][s_]:= Module[{cuv,t,aa,bb,abc,vbc,tt,u,v},
cuv[t_]:= {hu[t],hv[t]};
abc[t_] :=FullSimplify[D[cuv[tt],tt]/.tt->t,Element[t,Reals]];
If[pr==True,Print["cx'[s]= ",abc[s]]];
bb[t_] :=FullSimplify[fund1[ff][u,v][abc[t],abc[t]]/.{u->hu[t],v->hv[t]},Element[t,Reals]];
If[pr==True,Print["cx'.cx'  = ",bb[s]]];
vbc[t_] :=FullSimplify[abc[t]/Sqrt[bb[t]],Element[t,Reals]];
If[pr==True,Print["|cx'[t]| =",vbc[s]]];
aa[t_] := (fund1[ff][u,v][DD1[ff,cuv,vbc][t],DD1[ff,cuv,vbc][t]])/.{u->hu[t],v->hv[t]};
Sqrt[aa[s]/(bb[s])]];


rgt[f_][x__][i_,j_] := tv[f][i][x].tv[f][j][x];

RGT[f_][x__]:= Table[rgt[f][x][i,j],{i,1,Length[{x}]},{j,1,Length[{x}]}];


\!\(TraditionalForm\`fund1\)[f_][u__][x_List,y_List]:=
If[Length[x] == Length[{u}]&&Length[y] ==  Length[{u}],
Sum[rgt[f][u][i,j]*x[[i]]*y[[j]],{i,Length[{u}]},{j,Length[{u}]}],
Print["Wrong arguments!"];Return[]];

dV[ff_]:= Module[{m,i,j},m = Length[{##}];
Sqrt[Det[Table[rgt[ff][##][i,j],{i,m},{j,m}]]]]&;

dVol[gg_][u__]:= Module[{m,i,j,vg},m = Length[{u}];
vg=Det[Table[gg[u][i,j],{i,m},{j,m}]];Sqrt[Abs[vg]]];

christoffel1[j_,k_,i_][g_][x__]:= 
(D[g[x][i,j],{x}[[k]]] +D[g[x][j,k],{x}[[i]]] -D[g[x][k,i],{x}[[j]]])/2;

\[Gamma]\[Gamma][j_,k_,i_][ff_][x__] := Module[{g,ii,jj,y},
g[y__][ii_,jj_]:= rgt[ff][y][ii,jj];christoffel1[j,k,i][g][x]];

\[CapitalGamma][k_,i_,j_][g_]:=Sum[inverse[g][##][k,l]*christoffel1[l,i,j][g][##],{l,Length[{##}]}]&;

\[CapitalGamma]rgT[k_,i_,j_][ff_]:= Sum[invrgT[ff][##][k,l]*\[Gamma]\[Gamma][l,i,j][ff][##],{l,Length[{##}]}]&;

euclid[x__][i_,j_]:=\[Delta][i,j];



(* ::Subsubsection:: *)
(* Surfaces *)


(* tangent plane *)

tplane[f_][u0_,v0_][x_,y_]:= (f[u,v] + tangsp[f][u,v][x,y])/.{u->u0,v->v0};

(* Area element of a surface *)
dA[f_][u_,v_]:= Sqrt[ Det[Table[ rgt[f][u,v][i,j],{i,2},{j,2}]]];

(* Orthonormal moving frame of a surface *)
amf[1][f_][u_,v_]:= dotnorme[tv[f][1][u,v]]/;
(Head[f[u,v]] == List&&Length[f[u,v]]== 3);
amf[2][f_][u_,v_] := Cross[amf[3][f][u,v],amf[1][f][u,v]]/;
(Head[f[u,v]] == List&&Length[f[u,v]]== 3);
amf[3][f_][u_,v_]:= dotnorme[Cross[tv[f][1][u,v],tv[f][2][u,v]]]/;
(Head[f[u,v]] == List&&Length[f[u,v]]== 3);

fr[0][f_][u_,v_]:=Prepend[f[u,v],1];
fr[i_Integer][f_][u_,v_]:= Prepend[amf[i][f][u,v],0]/;(i>0)&&(i<4);

frame[f_][u_,v_] :=Transpose[Table[fr[j][f][u,v],{j,0,3}]];


(* ::Subsubsection::Closed:: *)
(*Exterior differential*)


dd[aa_][x__]:= Module[{ts=aa[x],ttp,he=Head[aa[x]], pp,di= Length[{x}],Arr,diff,dts},
If[Not[(he=== tensor)||(he=== List)]&&MemberQ[Attributes[aa],NumericFunction],
Return[d[aa][x]]];
If[he=== tensor,
ttp = type[ts];pp = ttp[[2,1]];
If[Not[ttp[[2,2]]== 0],Print["Tensor isn't covariant!"];Return[]];
If[Not[ttp[[3]]==di],Print["Dimensions differ!"];Return[]];
If[pp== 0,ts =aa[x][{},{}];Goto[diff]]];
If[he=== tensor,ts=tensorToArray[ts]; Goto[diff]];
Label[Arr];
pp=TensorRank[ts];
If[(pp==1)&&(Length[ Flatten[ts]]==1),pp=0;ts=Flatten[ts][[1]];Goto[diff]];
If[Not[TensorDimensions[ts][[1]]==di],Print["Dimensions differ!"];Return[]];
If[Not[TensorDimensions[ts]== Table[di,{pp}]],Print["Invalid Array!"];Return[]];
Label[diff];
If[(pp>1)&&(Not[(TensorSymmetry[ts] === Antisymmetric[Range[pp]])
||(TensorSymmetry[ts] === ZeroSymmetric[{}])]),
Print["Tensor ",aa[x]," is not antisymmetric!"];Return[]];
dts=((-1)^pp)*Simplify[(pp+1)*Normal[Symmetrize[D[ts,{{x}}],Antisymmetric[Range[pp+1]]]]];
If[(he===tensor)&&(pp>=di), Return[nulltensor[pp+1,0,di]]];
If[he===tensor, Return[ArrayTotensor[dts,pp+1]]];
dts];

dd/: dd[h1_+h2_][u__]:= dd[h1][u] +dd[h2][u];


(* ::Subsubsection::Closed:: *)
(*Cobasis and structure forms*)


\[Sigma][i_][f_][u__] :=If[(Head[f[u]]===List&&Length[f[u]]>=Length[{u}])&&(i>= 1)&&(i<= Length[f[u]]),
Return[Simplify[amf[i][f][u].d[f][u]]],
Print["Invalid argument!"];Return[]];

\[Omega][j_,k_][f_][u__] := Module[{n,m=Length[{u}]},
If[Head[f[u]]===List,n=Length[f[u]],Print["Invalid argument" ,f[u]," !"];Return[]];
If[m>n,Print["Invalid dimensions!"];Return[]];
If[j>=1&&j<=n&&k>=1&&k<=n,If[j==k,Return[null[m]],
Return[Simplify[amf[k][f][u].d[amf[j][f]][u]]]],Print["Invalid arguments!"];Return[]]];

(*Obsolet:
\[Omega][j_,k_][f_][u_,v_] := 
If[j>=1&&j<=3&&k>=1&&k<=3&&Length[f[u,v]]== 3,
Return[Simplify[amf[k][f][u,v].d[amf[j][f]][u,v]]],
Print["Invalid arguments!"];Return[]];
 *)
 
\[Omega][1][f_][u___]:= \[Omega][2,3][f][u]/;Length[f[u]]==3;
\[Omega][2][f_][u__]:= \[Omega][3,1][f][u]/;Length[f[u]]==3;
\[Omega][3][f_][u__]:= \[Omega][1,2][f][u]/;Length[f[u]]==3;

cg[1][ff_][u__] := dd[\[Sigma][1][ff]][u][[1,2]]/extprod[\[Sigma][1][ff][u], \[Sigma][2][ff][u]][[1,2]];
cg[2][ff_][u__] := dd[\[Sigma][2][ff]][u][[1,2]]/extprod[\[Sigma][1][ff][u], \[Sigma][2][ff][u]][[1,2]];


(* ::Subsubsection:: *)
(*Second fundamental form *)


fund2[ff_][u_,v_][x_List,y_List]:=
(d[ff][u,v].x).(d[amf[3][ff]][u,v].y);

fd2[1,1][ff_][u_,v_] := fund2[ff][u,v][{1,0},{1,0}];
fd2[1,2][ff_][u_,v_] := fund2[ff][u,v][{1,0},{0,1}];
fd2[2,2][ff_][u_,v_] := fund2[ff][u,v][{0,1},{0,1}];
fd2[2,1][ff_][u_,v_] := fund2[ff][u,v][{0,1},{1,0}];

cf[2,1][ff_][u_,v_]:= -extprod[\[Omega][1][ff][u,v],\[Sigma][2][ff][u,v]][[1,2]]/
extprod[\[Sigma][1][ff][u,v],\[Sigma][2][ff][u,v]][[1,2]];
cf[2,2][ff_][u_,v_]:= +extprod[\[Omega][1][ff][u,v],\[Sigma][1][ff][u,v]][[1,2]]/
extprod[\[Sigma][1][ff][u,v],\[Sigma][2][ff][u,v]][[1,2]];
cf[1,1][ff_][u_,v_]:= extprod[\[Omega][2][ff][u,v],\[Sigma][2][ff][u,v]][[1,2]]/
extprod[\[Sigma][1][ff][u,v],\[Sigma][2][ff][u,v]][[1,2]];
cf[1,2][ff_][u_,v_]:= extprod[\[Sigma][1][ff][u,v],\[Omega][2][ff][u,v]][[1,2]]/
extprod[\[Sigma][1][ff][u,v],\[Sigma][2][ff][u,v]][[1,2]];

(*normalcurvature *)
normalcurvature[ff_,h1_,h2_][t_]:= Module[{cvx,tt,vt,tx,aa,bb,u,v},
cvx[tt_] :={h1[tt],h2[tt]};
vt[tt_] := D[cvx[tx],tx]/.tx->tt;
aa = (-fund2[ff][u,v][vt[tt],vt[tt]]/.{u->h1[tt],v->h2[tt]})/.tt->t;
bb = (fund1[ff][u,v][vt[tt],vt[tt]]/.{u->h1[tt],v->h2[tt]})/.tt->t;
aa/bb];

(* Principal curvatures *)
prcurvatures[ff_][u_,v_] :=Eigenvalues[ Table[cf[i,j][ff][u,v],{i,2},{j,2}]];

(* Mean curvature 
meancurvature[ff_][u_,v_] := (cf[1,1][ff][u,v] + cf[2,2][ff][u,v])/2;*)

(* Mean curvature by A. Gray: *)
meancurvature[ff_][u_,v_] := (fd2[1,1][ff][u,v]*rgt[ff][u,v][2,2]-2*fd2[1,2][ff][u,v]*rgt[ff][u,v][1,2]+
fd2[2,2][ff][u,v]*rgt[ff][u,v][1,1])/(2*Det[Table[rgt[ff][u,v][i,j],{i,2},{j,2}]]);

(* Gauss curvature *)
gauss[ff_][u_,v_]:= Det[Table[fd2[i,j][ff][u,v],{i,2},{j,2}]]/
Det[Table[rgt[ff][u,v][i,j],{i,2},{j,2}]];

(* equivalent, but slower acting definitions:
gauss2[ff_][u_,v_] := 
extprod[\[Omega][1][ff][u,v],\[Omega][2][ff][u,v]][[1,2]]/
extprod[\[Sigma][1][ff][u,v],\[Sigma][2][ff][u,v]][[1,2]];

gauss3[ff_][u_,v_]:= Det[Table[cf[i,j][ff][u,v],{i,2},{j,2}]];
*)

(*normal congruence *)
normals[ff_, u_,v_][s_] := ff[u,v] + amf[3][ff][u,v]*s;


(* ::Subsubsection:: *)
(*Special surfaces *)


pluckerconoid[u_,v_] := {0,0, 2*Cos[v]*Sin[v]} + u*{Cos[v],Sin[v],0};

conesf[y_][u_,v_] := vec[y[v]]*u;

cylindersf[x_,y_][u_,v_] := {x[v],y[v],u};

tangsf[x_][u_,v_] := x[v] + D[x[v],v]*u;


(* ::Subsection:: *)
(*Manifolds*)


\[Delta][i_,j_] := KroneckerDelta[i,j];

(* var[x_,dd_:dimm] :=vec[x,dd]/.List-> Sequence; Obsolete! *)

var[x_,dd_:dimm] :=Module[{xx,n0=0},
If[Head[x]==Symbol,xx=vec[x,dd];n0=n0+1];If[Head[x]==List, xx= x;n0=n0+1];If[n0==0,Print["Invalid argument!"];Return[]];
xx/.List->Sequence];

imm[f_][x__] :=Table[f[i][x],{i,dimm}];

tx\[Iota][i_][x__] :=If[(i>=1)&&(i<=Length[{x}])&&IntegerQ[i], stb[i,Length[{x}]],
Print["Index ",i," invalid!"];Return[]];

X\[Iota][v_][x__]:= If[Length[v] == Length[{x}],Sum[v[[i]]*tx\[Iota][i][x],{i,Length[{x}]}],
Print["Arguments invalid!"];Return[]];

der[v_,ff_][x__]:=
If[(Head[v]===List)&&Length[v]==Length[{x}],Return[(d[ff][x]).v],
Print["Invalid arguments?"]; Return[]];

comm[v_,w_][x__] := If[(Length[v[x]]== Length[{x}])&&(Length[w[x]]== Length[{x}]),
Grad[w[x],{x}].v[x] -Grad[v[x],{x}].w[x], Print["Invalid arguments!"];Return[]];


(* Standard tangent basis *)


(* Tensor fields *)

 tensorfield[a_,p_,q_][x__]:= Module[{i,jj,n=Length[{x}]},
If[(p==0)&&(q==0),Return[tensor[a[x],0,0,n]]];
Global`b[ii_,jj_]:= a[ii,jj][x];
tensor[Global`b,p,q,n]];


(* Old def: tensorfield[a_,p_,q_][x__]:= Module[{bb,b,ii,jj,n=Length[{x}]},
If[(p==0)&&(q==0),b[{},{}]= a[x],b[ii_,jj_]:= a[ii,jj][x]];
bb=tensor[b,p,q,n];
bb]; *)



(* ::Subsection:: *)
(*Linear Connections*)


(* ::Subsubsection:: *)
(*Absolute differential*)


DD[g_][vw_][x__]:= Transpose[Simplify[
Table[D[vw[x][[k]],{x}[[j]]]+Sum[\[CapitalGamma][k,j,i][g][x]*vw[x][[i]],
{i,Length[{x}]}],{j,Length[{x}]},{k,Length[{x}]}]]];

DDimm[f_][vw_][x__]:= d[f][x].Transpose[Simplify[Table[D[vw[x][[k]],{x}[[j]]]+
Sum[\[CapitalGamma]rgT[k,i,j][f][x]*vw[x][[i]],{i,Length[{x}]}],{j,Length[{x}]},{k,Length[{x}]}]]];

DD1[ff_,cu_,vw_][t_] := Module[{mf,\[CapitalGamma]\[CapitalGamma],s, i,j,k,u,v},
mf = Length[cu[t]];
\[CapitalGamma]\[CapitalGamma][i_,j_,k_][s_]:=\[CapitalGamma]rgT[i,j,k][ff][u,v]/.{u-> cu[s][[1]],v->cu[s][[2]]};
D[vw[t],t] + Table[Sum[\[CapitalGamma]\[CapitalGamma][i,j,k][t]*vw[t][[j]]*D[cu[t][[k]],t],{j,mf},{k,mf}],{i,mf}]];

geodcurvature[ff_,hu_,hv_][s_]:= Module[{cuv,t,aa,bb,abc,vbc,tt},
cuv[t_]= {hu[t],hv[t]};
abc[t_] =D[cuv[tt],tt]/.tt->t;
bb[t_] = fund1[ff][hu[t],hv[t]][abc[t],abc[t]];
vbc[t_] =abc[t]/Sqrt[bb[t]];
aa[t_] = fund1[ff][hu[t],hv[t]][DD1[ff,cuv,vbc][t],DD1[ff,cuv,vbc][t]];
Sqrt[aa[s]/(bb[s])]];


cnf[\[Gamma]_][i_,j_][x__][v_]:=If[Not[MemberQ[Range[Length[{x}]],i]&&MemberQ[Range[Length[{x}]],j]], 
Print["Invalid indices ", i, ", ", j, " !"];Return[],
If[(Head[v]===List)&&(Length[{x}] == Length[v]),Sum[\[Gamma][j,h,i][x]*v[[h]],{h,Length[{x}]}],
Print["Invalid argument ", v, " !"];Return[]]];

nabla0[f_][x__]:=covecTotensor[d[f][x]];

nabla1[\[CapitalGamma]_,zz_][x__] := ArrayTotensor[Table[D[zz[x][[i]],{x}[[h]]]+ Sum[zz[x][[k]]*
\[CapitalGamma][i,h,k][x],{k,1,Length[{x}]}],{h, 1,Length[{x}]},{i,1,Length[{x}]}],1];

nab1[\[CapitalGamma]_,v_][x__][h_,i_] := D[v[x][[i]],{x}[[h]]] +Sum [v[x][[l]]*\[CapitalGamma][i,h,l][x],{l,Length[{x}]}];

nab[\[CapitalGamma]_, tens_][x__][ii_,ji_]:= Module[{di=Length[{x}],pt,qt},
(* Syntax control *)
If[Head[tens[x]]===tensor, pt = type[tens[x]][[2,1]];qt= type[tens[x]][[2,2]],
Print[tens[x]," is not a tensor!"];Return[]];
If[Not[MemberQ[Tuples[Range[di],pt+1],ii]],Print["Invalid argument ",ii;, " !"];Return[]];
If[Not[MemberQ[Tuples[Range[di],qt],ji]],Print["Invalid argument ",ji, " !"];Return[]];
If[Not[di == type[tens[x]][[3]]], 
Print["Tensor dimension ",type[tens[x]][[3]], " is not the manifold dimension", di," !"]; Return[]];
absD[\[CapitalGamma], tens][x][stb[ii[[pt+1]],di]][ii[[1;;pt]],ji]];

nabla[\[CapitalGamma]_, tens_][x__]:= Module[{di=Length[{x}],pt,qt},
(* Syntax control *)
If[Head[tens[x]]===tensor, pt = type[tens[x]][[2,1]];qt= type[tens[x]][[2,2]],
Print[tens[x]," is not a tensor!"];Return[]];
If[Not[di == type[tens[x]][[3]]], 
Print["Tensor dimension ",type[tens[x]][[3]], " is not the manifold dimension", di," !"]; Return[]];
(* The definition *)
tensor[nab[\[CapitalGamma], tens][x],pt+1,qt,di]];

absD[\[CapitalGamma]_,tens_][x__][v_][ii_,ji_]:= Module[{di=Length[{x}],pt,qt,wv,dt,dti,dtj,i,j,s,l},
(* Syntax control *)
If[Head[tens[x]]===tensor, pt = type[tens[x]][[2,1]];qt= type[tens[x]][[2,2]],
Print[tens[x]," is not a tensor!"];Return[]];
If[Not[MemberQ[Tuples[Range[di],pt],ii]],Print["Invalid argument ",ii;, " !"];Return[]];
If[Not[MemberQ[Tuples[Range[di],qt],ji]],Print["Invalid argument ",ji, " !"];Return[]];
If[Not[di == type[tens[x]][[3]]], 
Print["Tensor dimension ",type[tens[x]][[3]], " is not the manifold dimension", di," !"]; Return[]];
If[(Head[v] === List)&&Length[v] == di,
wv[i_,j_]:=Sum[\[CapitalGamma][j,s,i][x]*v[[s]],{s,Length[{x}]}],Print["Invalid argument ", v, " !"];Return[]];
(* The components *)
dt = Sum[D[tens[x][ii,ji],{x}[[s]]]*v[[s]],{s,di}];
dti =Sum[Sum[tens[x][Join[ii[[1;;l-1]],{s},ii[[l+1;;]]],ji]*wv[ii[[l]],s],{s,di}],{l,pt}];
dtj = Sum[Sum[tens[x][ii,Join[ji[[1;;l-1]],{s},ji[[l+1;;]]]]*wv[s,ji[[l]]],{s,di}],{l,qt}]; 
dt-dti+dtj];

tensorD[\[CapitalGamma]_,tens_][x__][v_]:= Module[{di=Length[{x}],pt,qt},
(* Syntax control *)
If[Head[tens[x]]===tensor, pt = type[tens[x]][[2,1]];qt= type[tens[x]][[2,2]],
Print[tens[x]," is not a tensor!"];Return[]];
If[Not[(Head[v] === List)&&Length[v] == di],
Print["Invalid argument ", v, " !"];Return[]];
tensor[absD[\[CapitalGamma],tens][x][v],pt,qt,di]];


(* ::Subsubsection::Closed:: *)
(*The absolute derivation of a tensor field along a curve*)


absD1[\[CapitalGamma]_,tens_][cu_,t_][ii_,ji_]:= Module[{di,pt,qt,wv,dt,dti,dtj,i,j,s,l,v,xt},
(* Syntax control *)
If[Head[cu[t]]===List, di = Length[cu[t]],Print[cu[t]," is not a curve!"];Return[]];
If[Head[tens[t]]===tensor, pt = type[tens[t]][[2,1]];qt= type[tens[t]][[2,2]],
Print[tens[t]," is not a tensor!"];Return[]];
If[Not[di == type[tens[t]][[3]]], 
Print["Tensor dimension ",type[tens[t]][[3]], " is not the manifold dimension", di," !"]; Return[]];
If[Not[MemberQ[Tuples[Range[di],pt],ii]],Print["Invalid argument ",ii, " !"];Return[]];
If[Not[MemberQ[Tuples[Range[di],qt],ji]],Print["Invalid argument ",ji, " !"];Return[]];
v = D[cu[t],t];xt =cu[t]/.List->Sequence;
wv[i_,j_]:=Sum[\[CapitalGamma][j,s,i][xt]*v[[s]],{s,di}];
(* The components *)
dt = D[tens[t][ii,ji],t];
dti =Sum[Sum[tens[t][Join[ii[[1;;l-1]],{s},ii[[l+1;;]]],ji]*wv[ii[[l]],s],{s,di}],{l,pt}];
dtj = Sum[Sum[tens[t][ii,Join[ji[[1;;l-1]],{s},ji[[l+1;;]]]]*wv[s,ji[[l]]],{s,di}],{l,qt}]; 
dt-dti+dtj];

tensorD1[\[CapitalGamma]_,tens_][cu_,t_]:= Module[{di,pt,qt},
(* Syntax control *)
If[Head[cu[t]]===List, di = Length[cu[t]],Print[cu[t]," is not a curve!"];Return[]];
If[Head[tens[t]]===tensor, pt = type[tens[t]][[2,1]];qt= type[tens[t]][[2,2]],
Print[tens[t]," is not a tensor!"];Return[]];
tensor[absD1[\[CapitalGamma],tens][cu,t],pt,qt,di]];


(* ::Subsubsection::Closed:: *)
(*The torsion tensor*)


tors[\[CapitalGamma]_][x__] := Module[{di = Length[{x}],to,i,j,k},
to[{i_,j_},{k_}]:=\[CapitalGamma][k,i,j][x]-\[CapitalGamma][k,j,i][x];
tensor[to,2,1,di]];


(* ::Subsubsection:: *)
(*The curvature tensor*)


RC[\[CapitalGamma]_][x__][k_,i_,h_,j_]:=D[\[CapitalGamma][k,j,i][x],{x}[[h]]]- D[\[CapitalGamma][k,h,i][x],{x}[[j]]]+
Sum[(\[CapitalGamma][k,h,l][x]\[CapitalGamma][l,j,i][x]-\[CapitalGamma][k,j,l][x]\[CapitalGamma][l,h,i][x]),{l,Length[{x}]}];

RCtensor[\[CapitalGamma]_][x__]:=Module[{arr,di=Length[{x}],k,h,i,j},
arr[{h_,i_,j_},{k_}]:=RC[\[CapitalGamma]][x][k,h,i,j];tensor[arr,3,1,di]];

Riccitensor[\[CapitalGamma]_][x__] := contraction[RCtensor[\[CapitalGamma]][x],2,1];

rct[\[CapitalGamma]_][x__][i_,j_]:= Sum[RC[\[CapitalGamma]][x][h,i,h,j],{h,Length[{x}]}];


(* ::Subsection:: *)
(*Riemannian Geometry*)


(* ::Subsubsection::Closed:: *)
(*A general symmetric, twofold covariant tensor field*)


symg[g_][x__][i_,j_]:=SymmetrizedArray[{k_,l_}->g[x][k,l],{Length[{x}],Length[{x}]},
Symmetric[{1,2}]][[i,j]];

rg[gg_:symg[Global`g]][x__][i_,j_]:=gg[x][i,j];

RG[gg_:rg[]][x__]:= If[
TensorSymmetry[Table[gg[x][i,j],{i,Length[{x}]},{j,Length[{x}]}]]== Symmetric[{1,2}],
Table[gg[x][i,j],{i,Length[{x}]},{j,Length[{x}]}], 
Print["Invalid argument ", gg," !"];Return[],Print["Invalid argument ", gg," !"];Return[]];

RGtensor[gg_:rg[]][x__]:=ArrayTotensor[RG[gg][x],2];


(* ::Subsubsection::Closed:: *)
(*Inverse metric tensor*)


INV[xx_][u__]:= Module[{m= Length[{u}]},
If[MatrixQ[xx[u]], Return[Inverse[xx[u]]]];
If[MatrixQ[xx[{u}]], Return[Inverse[xx[{u}]]]];
 Print["Wrong argument!"];Return[]];

inv[xx_][u__][i_,j_]:= INV[xx][u][[i,j]];


(* ::Subsubsection::Closed:: *)
(*The Pseudo-Euclidean spaces*)


eucl[u__][i_,j_] := KroneckerDelta[i,j];

EUCL[x__]:= RG[eucl][x];

lor[k_:indd][i_,j_]:=Module[{},If[Not[IntegerQ[i]]||Not[IntegerQ[j]]||(i<=0)||(j<=0),
Print["Invalid arguments!"];Return[]];
If[(i==j)&&(i<=k),-1,If[(i==j)&&(i>k),1,If[(i!=j),0]]]];

lorg[k_:indd][x__][i_,j_]:= lor[k][i,j];

 RGposk[r_][s__]:= Module[{n=Length[{s}],i,j,f,t,diag},
f[i_][t__] :=r^2* Product[(Cos[{t}[[j]]])^2,{j,i+1,n}];
diag [t__]:= Table[f[i][t],{i,1,n}]; 
DiagonalMatrix[diag[s]]];

rgposk[r_][s__][i_,j_]:= RGposk[r][s][[i,j]];



(* ::Subsubsection:: *)
(*The Christoffel symbols*)


chr[j_,k_,i_,g_:rg[]][x__]:=
(D[g[x][i,j],{x}[[k]]] +D[g[x][j,k],{x}[[i]]] -D[g[x][k,i],{x}[[j]]])/2;

\[CapitalGamma]g[g_:rg[]][k_,i_,j_][x__] := Module[{invg,gg,dd = Length[{x}],h,l,u},
If[Not[MemberQ[Tuples[Range[Length[{x}]],3],{k,i,j}]],
Print["Invalid indices ", k,",",i,",",j]; Return[]];
gg:=Table[g[x][h,l],{h,dd},{l,dd}];
invg = Inverse[gg];
Sum[invg[[k,h]]*chr[h,i,j,g][x], {h,dd}]];

\[CapitalGamma]f[ff_][h_,i_,j_][u__]:= \[CapitalGamma]g[rgt[ff]][h,i,j][u];

\[CapitalGamma]0[i_,j_,k_][x__] := 0;


(* ::Subsubsection:: *)
(*The covariant curvature tensor*)


covRCtensor[gg_:rg[]][x__]:= contraction[RGtensor[gg][x]\[CircleTimes]RCtensor[\[CapitalGamma]g[gg]][x],2,1];

covrc[gg_:rg[]][k_,h_,i_,j_][x__]:=covRCtensor[gg][x][{k,h,i,j},{}];

sectionalcurvature[gg_:rg[]][x__][X_, Y_]:=Module[{di=Length[{x}],tX,tY,gh,crt,nn,zz,i,j,k,l},
If[Not[(Head[X]=== List)&&(Head[Y]=== List)&&(Length[X]==di)&&(Length[Y]==di)],
Print["Invalid arguments ",X,Y," !"];Return[]];
If[MatrixRank[{X,Y}]<2,
Print[X,Y," are linearly dependent!"]; Return[]];
tX=vecTotensor[X];tY=vecTotensor[Y];
gh[i_,j_]:=gg[x][i,j];nn=tensorscpr[gh][tX\[Wedge]tY,tX\[Wedge]tY];
crt =covRCtensor[gg][x];
zz=Sum[crt[{i,j,k,l},{}]*X[[i]]*Y[[j]]*X[[k]]*Y[[l]],{i,di},{j,di},{k,di},{l,di}];
2*zz/nn];

R[k_,i_,h_,j_][g_][x__] := 
D[\[CapitalGamma][k,i,j][g][x],{x}[[h]]] - D[\[CapitalGamma][k,i,h][g][x],{x}[[j]]] - 
Sum[(\[CapitalGamma][k,l,j][g][x]*\[CapitalGamma][l,i,h][g][x]- \[CapitalGamma][k,l,h][g][x]*\[CapitalGamma][l,i,j][g][x]),{l,Length[{x}]}];


Rimm[k_,i_,h_,j_][ff_][x__] := 
D[\[CapitalGamma]rgT[k,i,j][ff][x],{x}[[h]]] -  D[\[CapitalGamma]rgT[k,i,h][ff][x],{x}[[j]]] - 
Sum[(\[CapitalGamma]rgT[k,l,j][ff][x]*\[CapitalGamma]rgT[l,i,h][ff][x]- \[CapitalGamma]rgT[k,l,h][ff][x]*\[CapitalGamma]rgT[l,i,j][ff][x]),{l,Length[{x}]}];

covR[k_,i_,h_,j_][g_][x__] :=  Sum[g[x][k,l]*R[l,i,h,j][g][x],{l,Length[{x}]}];

covRimm[k_,i_,h_,j_][ff_][x__] :=  Sum[rgt[ff][x][k,l]*Rimm[l,i,h,j][ff][x],{l,Length[{x}]}];


(* ::Subsubsection::Closed:: *)
(*Tensor Metric*)


tensorscpr[g_][ta_,tb_]:= Module[{pp,qq,nn,h,gg,hh,mg,i,j,s,t,tt,tta,ttb,tfa,tfb},
If[Not[(Head[ta]===tensor)&&(Head[tb] ===tensor)], 
Print["Arguments must be tensors!"];Return[]];
If[(type[ta][[2]]===type[tb][[2]])&&(type[ta][[3]]===type[tb][[3]]),
pp=type[ta][[2,1]];qq=type[ta][[2,2]];nn=type[ta][[3]],
Print["tensors belong to different spaces!"];Return[]];
If[pp==0,tta = ta;ttb=tb;Goto[SUM]];
mg=Table[g[i,j],{i,nn},{j,nn}];
h[i_,j_] :=Inverse[ mg][[i,j]];
hh[i_List,j_List]:=Product[h[i[[s]],j[[s]]],{s,pp}];
tfa[{},i_List] :=Sum[hh[i[[1;;pp]],tpl[nn,pp][[s]]]*ta[tpl[nn,pp][[s]],i[[pp+1;;pp+qq]]],
{s,nn^pp}];
tfb[{},i_List] :=Sum[hh[i[[1;;pp]],tpl[nn,pp][[s]]]*tb[tpl[nn,pp][[s]],i[[pp+1;;pp+qq]]],
{s,nn^pp}];
tta =tensor[tfa,0,pp+qq,nn];ttb = tensor[tfb,0,pp+qq,nn];
Label[SUM];
gg[i_List,j_List]:=Product[g[i[[s]],j[[s]]],{s,pp+qq}];
Sum[gg[tpl[nn,pp+qq][[t]],tpl[nn,pp+qq][[tt]]]*
tta[{},tpl[nn,pp+qq][[t]]]*ttb[{},tpl[nn,pp+qq][[tt]]],{t,nn^(pp+qq)},{tt,nn^(pp+qq)}]];

tensorfieldscpr[gg_:rg[]][ta_,tb_][x__]:=Module[{gh,i,j},
gh[i_,j_]:=gg[x][i,j];tensorscpr[gh][ta,tb]];


(* ::Subsubsection::Closed:: *)
(*The n - sphere. Elliptic Riemannian geometry*)


spher[r_][s__]:=r*unit[s];

sph[r_][x__] := r*invsterproj[{x}];

RGpos[r_][s__] := FullSimplify[RGT[spher[r]][s], Element[{s},Reals]];

RGcnfpos[r_][x__]:= (((2*r)/(1+ {x}.{x}))^2)*EUCL[x];

rgcnfpos[r_][i_,j_][x__]:= RGcnfpos[r][x][[i,j]]; 


(* ::Subsubsection::Closed:: *)
(*Hyperbolic Riemannian geometry*)


hypsp[r_][s__]:= Module[{n=Length[{s}],t},
If[n==1, Return[{r Cosh[s],r Sinh[s]}]];
t=Table[{s}[[i]],{i,2,n}];
r*Join[{Cosh[{s}[[1]]]},Evaluate[Sinh[{s}[[1]]]*unit[t]]]];

RGhypsp[r_][s__] := Module[{n=Length[{s}],i,j,rm},
rm:=FullSimplify[Table[pssp[tv[hypsp[r]][i][s],tv[hypsp[r]][j][s]],{i,n},{j,n}],
Element[{s},Reals]];
rm];

rghypsp[r_][s__][i_,j_]:= RGhypsp[r][s][[i,j]];

hypcnf[r_][s__]:= r*Join[{(1+{s}.{s})/(1-{s}.{s})},2*{s}/(1-{s}.{s})];

RGhypcnf[r_][s__] := Module[{n=Length[{s}],i,j,rm},
rm:=FullSimplify[Table[pssp[tv[hypcnf[r]][i][s],tv[hypcnf[r]][j][s]],{i,n},{j,n}],
Element[{s},Reals]];rm];

rghypcnf[r_][s__][i_,j_]:= RGhypcnf[r][s][[i,j]];

RGneg[r_][s__] := (2*r/(-1+{s}.{s}))^2*IdentityMatrix[Length[{s}]];

rgneg[r_][s__][i_,j_]:= RGneg[r][s][[i,j]];


(* ::Section:: *)
(*The End*)


End[]

Protect["`*"];

EndPackage[]




